package com.lzmispt.data.resource.service.monitor.impl;

package .tmp;

import com.lzmispt.data.center.entity.dto.databasestruct.DataSourceDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.SysFieldDTO;
import com.lzmispt.data.center.entity.enums.BaseSysFieldEnum;
import com.lzmispt.data.center.entity.enums.FileSysFieldEnum;
import com.lzmispt.data.center.entity.vo.resource.datasource.DataSourceVO;
import com.lzmispt.data.center.entity.vo.statistics.CommonDataCountVO;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.dao.file.IFileInfoFieldDao;
import com.lzmispt.data.resource.dao.file.IFileInfoTableDao;
import com.lzmispt.data.resource.dao.metadata.IMetadataDao;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataEntityDTO;
import com.lzmispt.data.resource.entity.po.file.FileInfoTablePO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityPO;
import com.lzmispt.data.resource.manager.metadata.MetadataManager;
import com.lzmispt.data.resource.service.monitor.IPlatCommonDataQueryService;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.databasestruct.TypeConverter;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.sysfield.SysFieldHelper;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.log.LogHelper;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

public abstract class PlatDataQueryManager implements IPlatCommonDataQueryService {
  protected static final String DEFAULT_TOKEN = "0";
  
  @Autowired
  DataStandardClient dataStandardClient;
  
  public CommonDataCountVO getCommonDataCount(List<Long> entityIds) {
    CommonDataCountVO commonDataCountVO = new CommonDataCountVO();
    commonDataCountVO.setClassifyCount(getClassifyCount());
    commonDataCountVO.setTotalDataTableCount(null);
    commonDataCountVO.setTotalDataTableFieldCount(null);
    List<Long> allEntityIds = getTotalFileTableCount();
    List<Long> allTableFileIdList = getTotalFileTableFieldCount();
    if (!CollectionUtils.isEmpty(entityIds) && !CollectionUtils.isEmpty(allEntityIds))
      allEntityIds = (List<Long>)allEntityIds.stream().filter(entityIds::contains).collect(Collectors.toList()); 
    if (!CollectionUtils.isEmpty(entityIds) && !CollectionUtils.isEmpty(allTableFileIdList))
      allTableFileIdList = (List<Long>)allTableFileIdList.stream().filter(entityIds::contains).collect(Collectors.toList()); 
    commonDataCountVO.setTotalFileTableCount(Integer.valueOf((allEntityIds != null) ? allEntityIds.size() : 0));
    commonDataCountVO.setTotalFileTableFieldCount(Integer.valueOf((allTableFileIdList != null) ? allTableFileIdList.size() : 0));
    return commonDataCountVO;
  }
  
  public Integer getTotalDataCount() {
    LogHelper.info("开始查询 %s 平台数据总量", new Object[] { getDefaultCode() });
    Map<Long, DataStructProvider> dataStructProviderCache = new HashMap<>();
    Set<String> handledTable = new HashSet<>();
    int totalCount = 0;
    try {
      List<MetadataEntityPO> entityList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getAllMetadataEntities(null);
      for (MetadataEntityPO entity : entityList) {
        if (entity.getDataSourceId() == null)
          continue; 
        Integer count = queryTableDataCount(entity.getEntityTableName(), entity.getDataSourceId(), 
            SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE), dataStructProviderCache, handledTable);
        if (count != null) {
          LogHelper.info("数据表 %s 数据量为 %d", new Object[] { entity.getEntityTableName(), count });
          totalCount += count.intValue();
        } 
      } 
      List<FileInfoTablePO> fileTableList = ((IFileInfoTableDao)ServiceUtil.getFileInfoTableDao(getDefaultCode(), IFileInfoTableDao.class)).getAllFileInfoTable();
      for (FileInfoTablePO fileTable : fileTableList) {
        if (fileTable.getDataSourceId() == null)
          continue; 
        Integer count = queryTableDataCount(fileTable.getFileTableName(), fileTable.getDataSourceId(), 
            SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.IS_DELETE), dataStructProviderCache, handledTable);
        if (count != null) {
          LogHelper.info("文件表 %s 数据量为 %d", new Object[] { fileTable.getFileTableName(), count });
          totalCount += count.intValue();
        } 
      } 
    } catch (Exception e) {
      LogHelper.error(MessageFormat.format("查询{0}数据总量异常", new Object[] { getDefaultCode() }), ExceptionUtil.getStackTraceMsg(e));
    } 
    return Integer.valueOf(totalCount);
  }
  
  public Integer getDataTableDataCountById(Long entityId, Integer res, String tokenId) {
    ResAndDataEntityDTO entity = MetadataManager.selectIsMasterMetadataEntityByDirectoryId(entityId, res, getDefaultCode(), tokenId);
    if (entity.getDataSourceId() == null || StringUtils.isEmpty(entity.getEntityTableName())) {
      LogHelper.info("tableId: " + entityId + " 未配置数据源或表名为空，无法查询数据量");
      return null;
    } 
    return queryTableDataCount(entity.getEntityTableName(), entity.getDataSourceId(), SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE), new HashMap<>(), new HashSet<>());
  }
  
  public Integer queryTableDataCount(String tableName, Long dataSourceId, String isDeleteFieldName, Map<Long, DataStructProvider> dataStructProviderCache, Set<String> handledTable) {
    DataStructProvider dataStructProvider = simpleCacheGet(dataStructProviderCache, dataSourceId, () -> {
          DataSourceVO dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptByIdNotTokenId(dataSourceId, getDefaultCode()).getObj();
          if (dataSourceVO == null) {
            LogHelper.info("未查询到id: " + dataSourceId + " 的数据源，跳过");
            return null;
          } 
          DataSourceDTO dataSourceDTO = new DataSourceDTO();
          BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
          ResultEntity<DataStructProvider> dataBaseProvider = DataStructProvider.getDataBaseProvider(dataSourceDTO);
          if (dataBaseProvider.getCode().intValue() != 0)
            LogHelper.error("获取id:" + dataSourceId + " 的数据源失败,原因：" + dataBaseProvider.getMsg()); 
          return (DataStructProvider)dataBaseProvider.getObj();
        });
    if (dataStructProvider == null)
      return null; 
    String tableIdentity = assembleTableIdentity(dataStructProvider.currentDataSource, tableName);
    if (handledTable.contains(tableIdentity))
      return null; 
    handledTable.add(tableIdentity);
    return selectTableDataCount(tableName, isDeleteFieldName, dataStructProvider);
  }
  
  protected String assembleTableIdentity(DataSourceDTO dataSource, String tableName) {
    return String.format("%s-%s-%d-%s-%s-%s", new Object[] { dataSource.getType(), dataSource.getIpName(), dataSource.getPort(), dataSource.getDbName(), dataSource.getDbSchema(), tableName });
  }
  
  public Integer selectTableDataCount(String tableName, String isDeleteFieldName, DataStructProvider dataStructProvider) {
    try {
      String sql = "select count(*) from " + TypeConverter.getColumnSqlStr(tableName, dataStructProvider.getDbType()) + " where " + TypeConverter.getColumnSqlStr(isDeleteFieldName, dataStructProvider.getDbType()) + " = 0";
      return dataStructProvider.execCountSql(sql, new HashMap<>());
    } catch (Exception e) {
      LogHelper.error(MessageFormat.format("获取 {} 表中数量总数失败", new Object[] { tableName }), ExceptionUtil.getStackTraceMsg(e));
      return null;
    } 
  }
  
  public Integer getClassifyCount() {
    return ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getClassifyCount();
  }
  
  public Integer getTotalDataTableCount() {
    return ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getTotalDataTableCount();
  }
  
  public Integer getTotalDataTableFieldCount() {
    List<String> baseSysFieldNames = (List<String>)SysFieldHelper.getBaseSysFields().values().stream().map(SysFieldDTO::getFieldName).distinct().collect(Collectors.toList());
    return ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).getTotalDataTableFieldCount(baseSysFieldNames);
  }
  
  public List<Long> getTotalFileTableCount() {
    return ((IFileInfoTableDao)ServiceUtil.getFileInfoTableDao(getDefaultCode(), IFileInfoTableDao.class)).getTotalFileTableCount();
  }
  
  public List<Long> getTotalFileTableFieldCount() {
    List<String> fileSysFieldNames = (List<String>)SysFieldHelper.getFileSysFields().values().stream().map(SysFieldDTO::getFieldName).distinct().collect(Collectors.toList());
    return ((IFileInfoFieldDao)ServiceUtil.getFileInfoFieldDao(getDefaultCode(), IFileInfoFieldDao.class)).getTotalFileTableFieldCount(fileSysFieldNames);
  }
  
  public static <K, T> T simpleCacheGet(Map<K, T> cache, K key, Supplier<T> queryFunc) {
    if (cache.containsKey(key))
      return cache.get(key); 
    T result = queryFunc.get();
    cache.put(key, result);
    return result;
  }
  
  public String getTableNameById(String tableId) throws Exception {
    ResAndDataEntityDTO resAndDataEntityDTO = MetadataManager.selectIsMasterMetadataEntityByDirectoryId(Long.valueOf(tableId), DCConst.DataResourceCategory.DATA, getDefaultCode(), "0");
    if (ObjectUtils.isEmpty(resAndDataEntityDTO))
      return ""; 
    return resAndDataEntityDTO.getEntityName();
  }
  
  public String setTableId(Integer tablePlatCode, String tableId) {
    return getDefaultCode() + "_" + tableId;
  }
}
